from fastapi import Depends
from fastapi_utils.inferring_router import InferringRouter

from models.base import (
    HTTP_500_RESPONSE_MODEL,
    HTTP_401_RESPONSE_MODEL,
)

from models.rds import (
    RdsLogoutModel,
    RdsRemoteAppModel,
    RdsUserModel,
    RemoteAppUsersModel,
    CheckInstanceResponse,
    SessionsResponse,
    LoginResponse,
    LogoutUserResponse,
    TestConnectResponse,
    CollectionsResponse,
    RdsRemoteAppResponse,
    RemoteAppUsersResponse,
    CollectionInfoResponse
)

from utils.rds import check_instance_is_rds
from services.rds import RdsService
from utils.rds import check_instance_is_rds
from container import Container
from dependency_injector.wiring import Provide, inject
from core.secure import check_auth

rds_router = InferringRouter()


@rds_router.get("/rds/test", response_model=CheckInstanceResponse,
                responses={**HTTP_500_RESPONSE_MODEL,})
def check_rds_instance():
    """method for handling server rds check request.

    Returns:
        response (dict): json response
    """

    response = check_instance_is_rds()

    return response


@rds_router.get("/rds/sessions",
                response_model=SessionsResponse,
                dependencies=[Depends(check_auth)],
                responses={**HTTP_500_RESPONSE_MODEL,
                           **HTTP_401_RESPONSE_MODEL,})
@inject
def rds_sessions(service: RdsService = Depends(Provide[Container.service])):
    """method for handling a request to get rds user sessions.

    Returns:
        response (dict): json response
    """

    return service.get_sessions()


@rds_router.post("/rds/login", response_model=LoginResponse,
                 responses={**HTTP_401_RESPONSE_MODEL,})
@inject
def rds_login(params: RdsUserModel, service: RdsService = Depends(Provide[Container.service])):
    """method to handle connection request to rds.

    Args:
        params (RdsUserModel): auth params

    Returns:
        response (dict): json response
    """

    return service.login(params)


@rds_router.post("/rds/testconnect", response_model=TestConnectResponse,
                 responses={**HTTP_401_RESPONSE_MODEL,})
@inject
def rds_test_connect(params: RdsUserModel, service: RdsService = Depends(Provide[Container.service])):
    """method for handling request for a test connection to RDS.

    Args:
        params (RdsUserModel): auth params

    Returns:
        response (dict): json response
    """

    return service.test_connect(params)


@rds_router.post("/rds/disconnect",
                 response_model=LogoutUserResponse,
                 dependencies=[Depends(check_auth)],
                 responses={**HTTP_500_RESPONSE_MODEL,
                            **HTTP_401_RESPONSE_MODEL,})
@inject
def rds_logout_user(params: RdsLogoutModel, service: RdsService = Depends(Provide[Container.service])):
    """method for handling a request to disconnect a user.

    Args:
        params (RdsLogoutModel): auth params

    Returns:
        response (dict): json response
    """

    return service.logout_user(params)


@rds_router.get("/rds/collections",
                response_model=CollectionsResponse,
                dependencies=[Depends(check_auth)],
                responses={**HTTP_500_RESPONSE_MODEL,
                           **HTTP_401_RESPONSE_MODEL,})
@inject
def rds_collections(service: RdsService = Depends(Provide[Container.service])):
    """method for handling a request to get rds collections.

    Returns:
        response (dict): json response
    """

    return service.get_collections()


@rds_router.post("/rds/apps",
                 response_model=RdsRemoteAppResponse,
                 dependencies=[Depends(check_auth)],
                 responses={**HTTP_500_RESPONSE_MODEL,
                            **HTTP_401_RESPONSE_MODEL,})
@inject
def rds_apps(params: RdsRemoteAppModel, service: RdsService = Depends(Provide[Container.service])):
    """method for handling a request to get published remote apps.

    Args:
        params (RdsRemoteAppModel): contains collection name

    Returns:
        response (dict): json response
    """

    return service.get_remote_apps(params)


@rds_router.post("/rds/apps/users",
                 response_model=RemoteAppUsersResponse,
                 dependencies=[Depends(check_auth)],
                 responses={**HTTP_500_RESPONSE_MODEL,
                            **HTTP_401_RESPONSE_MODEL,})
@inject
def rds_app_users(params: RemoteAppUsersModel, service: RdsService = Depends(Provide[Container.service])):
    """method for handling a request to getting users that have access
     to published app by collection name and remote app alias.

    Args:
        params (RemoteAppUsersModel): contains collection name
                                    and remote app alias

    Returns:
        response (dict): json response
    """

    return service.get_remote_app_users(params)


@rds_router.get("/rds/collection_info/{collection_name}",
                response_model=CollectionInfoResponse,
                dependencies=[Depends(check_auth)],
                responses={**HTTP_500_RESPONSE_MODEL,
                           **HTTP_401_RESPONSE_MODEL,})
@inject
def collection_info(collection_name: str, service: RdsService = Depends(Provide[Container.service])):
    """method for handling a request to get collection info by collection name

    Returns:
        response (dict): json response
    """

    return service.get_collection_info(collection_name)
