﻿import os
import eventlog

from pywintypes import error as pywin_error
import win32security
import win32api
from wmi import x_wmi

from constants.windows.constants import ConstRds
from excepts import OSAPIError, AuthenticationError
from utils.rds import (
    psh_request_handler,
    WMIHandler,
)

from models.rds import (
    RdsUserModel,
    RdsLogoutModel,
    RdsRemoteAppModel,
    RemoteAppUsersModel,
)


class RdsService:
    """
    Сервис взаимодействия с RDS

    Handled errors:
    - OSAPIError (из модуля excepts): the error that occurs
      when executing a request API ОС
    - AuthenticationError (из модуля excepts): the error that occurs
      during authentication
    """

    def __init__(
            self,
            wmi_handler: WMIHandler,
            const: ConstRds):
        self._wmi_handler = wmi_handler
        self.GET_REMOTE_APP_USERS_CMD = const.GET_REMOTE_APP_USERS_CMD
        self.LOGOFF_USER_CMD = const.LOGOFF_USER_CMD

    def get_sessions(self) -> dict:
        """function returning active rds sessions.

        Returns:
            result (dict): json response
        """
        try:
            sessions = self._wmi_handler.get_sessions()
            result = {"sessions": sessions}

        except x_wmi as ex:
            raise OSAPIError(ex)

        return result

    def login(self, params: RdsUserModel) -> dict:
        """authorization function in rds

        Args:
            params (RdsInputModel): auth params

        Returns:
            result (dict): json response
        """

        try:
            # LogonUser() return a handle to a token that represents the specified user.
            # Returned handle is a primary token that you can use in calls to the
            # CreateProcessAsUser() function.
            # If you specify the LOGON32_LOGON_NETWORK flag, LogonUser() returns
            # an impersonation token that you cannot use in CreateProcessAsUser()
            # unless you call DuplicateTokenEx() to convert it to a primary token.
            # LOGON32_PROVIDER_WINNT40: Use the NTLM logon provider.
            token = win32security.LogonUser(
                params.login,
                params.domain,
                params.password,
                win32security.LOGON32_LOGON_NETWORK,
                win32security.LOGON32_PROVIDER_WINNT40,
            )
            authenticated = bool(token)
            win32api.CloseHandle(token)

            result = dict(auth_done=authenticated)
            eventlog.write_event("Auth is done.", "INFO")
        except pywin_error as ex:
            raise AuthenticationError(ex)

        return result

    def test_connect(self, params: RdsUserModel) -> dict:
        """method that return response of trying to connect to rds

        Args:
            params (RdsUserModel): connect params

        Returns:
            result (dict): json response
        """

        # TODO Пока не понятно как правильно делать проверку тестового подключения по RDP.
        # Оставим текущую логику закомментированной.

        # RDS_HOSTNAME = '192.168.3.13'
        # login = params.login
        # pwd = params.password

        # os.system(
        #     'cmd/c "cmdkey /generic:"{hostname}" /user:"{login}" \
        #         /pass:"{pwd}""'.format(hostname=RDS_HOSTNAME, login=login, pwd=pwd)
        #     )
        # os.system('cmd/c "mstsc /v:{hostname}"'.format(hostname=RDS_HOSTNAME))
        # os.system('cmd/c "cmdkey /delete:{hostname}"'.format(hostname=RDS_HOSTNAME))

        # Временное решение делать проверку посредством попытки авторизации на AD.
        self.login(params)
        result = {"is_connected": True}
        return result

    def logout_user(self, params: RdsLogoutModel) -> dict:
        """method disconnect user from RDS

        Args:
            params (RdsInputModel): logout params

        Returns:
            result (dict): json response
        """
        try:
            sessions = self._wmi_handler.get_sessions()
            session_id = params.session_id
            # In future use host server
            # Example: logoff /VM <session id> /server:<host server>
            session_found = False
            for session in sessions:
                if session["session_id"] == session_id:
                    session_found = True
                    os.system(
                        'cmd/c "echo y|LOGOFF /VM {session_id}"'.format(session_id=session_id)
                    )
                    break

            if session_found:
                result = dict(disconnect_done=True)
                eventlog.write_event(f"Logout user {session_id}", "INFO")
            else:
                result = dict(disconnect_done=False)  # Session ID not found
                eventlog.write_event("User is not disconnected. Session ID not found", "ERROR")

        except Exception as ex:
            raise OSAPIError(ex)

        return result

    def get_collections(self) -> dict:
        """Getting RDS collections.

        Returns:
            result (dict): json response
        """
        try:
            collections = self._wmi_handler.get_collections()
            result = dict(collections=collections)

        except x_wmi as ex:
            raise OSAPIError(ex)

        return result

    def get_remote_app_users(self, params: RemoteAppUsersModel) -> dict:
        """Getting users that have access to published
         app by collection name and remote app alias.

        Args:
            params (RemoteAppUsersModel): contains collection name
                                        and remote app alias

        Returns:
            result (dict): json response
        """
        try:
            collection = params.collection
            app_alias = params.app_alias
            # Getting users list using powershell handler. Todo: soon to be replaced with WMI handler.
            output = psh_request_handler(
                self.GET_REMOTE_APP_USERS_CMD.format(collection, app_alias)
            )  # FIXME[aap]: В текущей реализации невозможно понять, что команда завершилась с ошибкой
            app_users = [users.lstrip("UserGroups : ") for users in output]
            result = dict(app_users=app_users)

        except Exception as ex:
            raise OSAPIError(ex)

        return result

    def get_remote_apps(self, params: RdsRemoteAppModel) -> dict:
        """Getting published remote apps from RDS by collection name.

        Args:
            params (RdsRemoteAppModel): contains collection name
            wmi_handler

        Returns:
            result (dict): json response
        """
        try:
            collection = params.collection
            output = self._wmi_handler.get_remote_apps(collection)
            result = dict(apps=output)

        except x_wmi as ex:
            raise OSAPIError(ex)

        return result

    def get_collection_info(self, collection_name: str):
        """Getting collection info by collection name

        Args:
            collection_name (str): contains collection name

        Returns:
            result (dict): json response
        """
        try:
            collection_info = self._wmi_handler.get_collection_info(collection_name)

            result = dict(collection_info=collection_info)
        except x_wmi as ex:
            raise OSAPIError(ex)

        return result
